﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TicTacToe
{
    public class Game
    {
        private static readonly int[][] allVictories = new int[][]
        {
            new int[] {0, 1, 2},
            new int[] {3, 4, 5},
            new int[] {6, 7, 8},
            new int[] {0, 3, 6},
            new int[] {1, 4, 7},
            new int[] {2, 5, 8},
            new int[] {0, 4, 8},
            new int[] {2, 4, 6}
        };

        private string _victor;

        private List<int[]> crossVictories;
        private List<int[]> circleVictories;

        private Tile[,] field;

        public string Cross { get; private set; }
        public string Circle { get; private set; }

        public bool Tie
        {
            get
            {
                return ((crossVictories.Count == 0) && (circleVictories.Count == 0));
            }
        }

        public string Victor
        {
            get
            {
                return _victor;
            }
        }

        public Tile this[int x, int y]
        {
            get
            {
                return field[x, y];
            }
            set
            {
                field[x, y] = value;

                if (value == Tile.Cross)
                {
                    if (crossVictories.Any(victory => victory.All(i => (field[(i % 3), (i / 3)]) == Tile.Cross)))
                    {
                        _victor = Cross;
                    }

                    circleVictories = circleVictories.Where(v => !v.Contains((x + (3 * y)))).ToList();
                }
                else if (value == Tile.Circle)
                {
                    if (circleVictories.Any(victory => victory.All(i => (field[(i % 3), (i / 3)]) == Tile.Circle)))
                    {
                        _victor = Circle;
                    }

                    crossVictories = crossVictories.Where(victory => !victory.Contains((x + (3 * y)))).ToList();
                }
            }
        }

        public Game(string cross, string circle)
        {
            _victor = null;

            crossVictories = new List<int[]>();
            circleVictories = new List<int[]>();

            crossVictories.AddRange(allVictories);
            circleVictories.AddRange(allVictories);

            field = new Tile[3, 3];

            Cross = cross;
            Circle = circle;
        }
    }
}
