﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TicTacToe
{
    public partial class MainForm : Form
    {
        private static readonly Pen FieldPen = new Pen(Color.Black, 4);

        private Game game;

        private NewGameDialog dialog;

        private bool turn;

        public MainForm()
        {
            game = null;

            dialog = new NewGameDialog();

            InitializeComponent();

            Paint += DrawGame;
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                game = dialog.Game;

                turn = true;

                player1Label.Visible = true;
                player2Label.Visible = true;

                player1Label.Text = game.Cross;
                player2Label.Text = game.Circle;

                RefreshGame();
            }
        }

        private void RefreshGame()
        {
            if (turn)
            {
                player1Label.ForeColor = Color.Black;
                player2Label.ForeColor = Color.Gray;
            }
            else
            {
                player2Label.ForeColor = Color.Black;
                player1Label.ForeColor = Color.Gray;
            }

            Invalidate();
        }

        private void DrawGame(object sender, PaintEventArgs e)
        {
            if (game == null)
            {
                return;
            }

            for (int i = 0; i < 4; i++)
            {
                e.Graphics.DrawLine(FieldPen, 50, (150 + (i * 100)), 350, (150 + (i * 100)));
                e.Graphics.DrawLine(FieldPen, (50 + (i * 100)), 150, (50 + (i * 100)), 450);
            }

            for (int x = 0; x < 3; x++)
            {
                for (int y = 0; y < 3; y++)
                {
                    switch (game[x, y])
                    {
                        case Tile.Circle:
                            {
                                e.Graphics.DrawCircle((60 + (x * 100)), (160 + (y * 100)), 80);
                                break;
                            }
                        case Tile.Cross:
                            {
                                e.Graphics.DrawCross((60 + (x * 100)), (160 + (y * 100)), 80);
                                break;
                            }
                        default: break;
                    }
                }
            }
        }

        private void MainForm_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left)
            {
                return;
            }

            if ((e.X > 50) && (e.X < 350) && (e.Y > 150) && (e.Y < 450))
            {
                int x = ((e.X - 50) / 100);
                int y = ((e.Y - 150) / 100);

                if (game[x, y] == Tile.Empty)
                {
                    if (turn)
                    {
                        game[x, y] = Tile.Cross;
                    }
                    else
                    {
                        game[x, y] = Tile.Circle;
                    }

                    RefreshGame();

                    turn = !turn;

                    string gameEndsMessage = null;

                    if (game.Tie || (game.Victor != null))
                    {
                        gameEndsMessage = "The Game ends with a tie!";

                        if (!game.Tie)
                        {
                            gameEndsMessage = String.Format("Congratulations! Player {0} won the game!", game.Victor);
                        }

                        game = null;

                        player1Label.Visible = false;
                        player2Label.Visible = false;
                    }

                    if (gameEndsMessage != null)
                    {
                        MessageBox.Show(gameEndsMessage, "Game Over");
                    }

                    RefreshGame();
                }

            }
        }
    }
}
